#include "stm32_time.h"

static __IO uint32_t timing_delay;
uint32_t clocks_per_us;

/*******************************************************************************
* Function Name  : SysTickHandler
* Description    : This function handles SysTick Handler.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void SysTick_Handler(void)
{
  if(timing_delay!=0)
    timing_delay--;
}

// public functions
void time_init(void)
{
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

  // set the time base to 1ms
  SysTick_Config(SystemCoreClock / 1000);
  NVIC_SetPriority(SysTick_IRQn,0);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1,ENABLE);

  TIM_TimeBaseStructure.TIM_Period = 0xFFFF;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_RepetitionCounter=0x0000;
  TIM_TimeBaseInit(TIM1,&TIM_TimeBaseStructure);
  TIM_SetCounter(TIM1,0);

  clocks_per_us=(SystemCoreClock/1000000)+1;
}

void delay_ms(__IO uint32_t time)
{
  timing_delay=time;

  while(timing_delay!=0);
}

void delay_us(uint32_t us) // microseconds
{
  uint32_t clocks,current;

  clocks=clocks_per_us*us;
  TIM_SetCounter(TIM1,0);
  TIM_Cmd(TIM1, ENABLE);
  do{
    current=TIM_GetCounter(TIM1);
  }while(current<clocks);
  TIM_Cmd(TIM1, DISABLE);
}

